/*
 *  Seguidor de envolvente
 *
 *  Avelino Herrera para "Slo Programadores"

	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>

/*  Frecuencia de muestreo de salida. */
#define  SAMPLE_RATE           44100

#ifdef M_PI
#define  PI  M_PI
#else
#define  PI  3.1415926
#endif

/*  Devuelve el nmero de muestras que hay en m milisegundos. */
#define  GET_SAMPLES_MS(m)  ((m * SAMPLE_RATE) / 10000)
/*  Devuelve el nmero de muestras que hay en m centsimas de segundo. */
#define  GET_SAMPLES_CS(m)  ((m * SAMPLE_RATE) / 100)

#define  BUFFER_SIZE  256
#define  ENVELOPE_FOLLOWER_NUM_POINTS  2000


typedef struct {
	float buffer[ENVELOPE_FOLLOWER_NUM_POINTS];
	float *p;
	float *p_max;
	int buffer_size;
	float val;
	} envfollower;


void envfollower_init(envfollower *ef, int buffer_size) {
	memset(ef->buffer, 0, ENVELOPE_FOLLOWER_NUM_POINTS * sizeof(float));
	ef->buffer_size = buffer_size;
	ef->p = ef->buffer;
	ef->p_max = ef->buffer + buffer_size - 1;
	ef->val = 0;
	return;
	}

void envfollower_work(envfollower *ef, float *input, float *output) {
	int i;

	for (i = 0; i < BUFFER_SIZE; i++) {
		ef->val -= *(ef->p);
		if (input[i] < 0)
			*(ef->p) = -input[i];
		 else
			*(ef->p) = input[i];
		ef->val += *(ef->p);
		ef->p++;
		if (ef->p == ef->p_max)
			ef->p = ef->buffer;
		output[i] = ef->val / (float)ef->buffer_size;
		if (output[i] < 0)
			output[i] = 0.0;
		}
	return;
	}

void envfollower_done(envfollower *ef) {
	return;
	}

int buffer_rec(int fd, float *left, float *right) {
	int i, readed;
	short int temp_buffer[BUFFER_SIZE * 2];

	readed = read(fd, temp_buffer, BUFFER_SIZE * 2 * sizeof(short int));
	for (i = 0; i < BUFFER_SIZE; i++) {
		left[i] = ((float) temp_buffer[2 * i]) / 32768.0;
		right[i] = ((float) temp_buffer[(2 * i) + 1]) / 32768.0;
		}
	if (readed < (BUFFER_SIZE * 2 * sizeof(short int)))
		return 0;
	return 1;
	}


void buffer_play(int fd, float *left, float *right, float gain) {
	int i;
	int trash;
	short int temp_buffer[BUFFER_SIZE * 2];

	for (i = 0; i < BUFFER_SIZE; i++) {
		left[i] *= gain;
		if (left[i] > 1.0)
			left[i] = 1.0;
		else if (left[i] < -1.0)
			left[i] = -1.0;
		right[i] *= gain;
		if (right[i] > 1.0)
			right[i] = 1.0;
		else if (right[i] < -1.0)
			right[i] = -1.0;
		temp_buffer[2 * i] = (short) rint(left[i] * 32767);
		temp_buffer[(2 * i) + 1] = (short) rint(right[i] * 32767);
		}
	write(fd, temp_buffer, BUFFER_SIZE * 2 * sizeof(short int));
	return;
	}


int audio_open(void) {
	int rate = SAMPLE_RATE;
	int format = AFMT_S16_LE;
	int stereo = 1;
	int fd;

	if ((fd = open("/dev/dsp", O_WRONLY, 0)) == -1) {
		perror("/dev/dsp");
		exit(-1);
		}
	ioctl(fd, SNDCTL_DSP_SPEED, &rate);
	ioctl(fd, SNDCTL_DSP_SETFMT, &format);
	ioctl(fd, SNDCTL_DSP_STEREO, &stereo);
	return fd;
	}


/*****************************************************************************/
int main(int argc, char *argv[]) {
	envfollower ef;
	float left_input[BUFFER_SIZE], right_input[BUFFER_SIZE];
	float left_output[BUFFER_SIZE], right_output[BUFFER_SIZE];
	float env_output[BUFFER_SIZE];
	int i;
	int fd;

	fd = audio_open();
	envfollower_init(&ef, 2000);
	/* Leemos muestras de la entrada estndar */
	while (buffer_rec(0, left_input, right_input)) {
		/* Detectamos la envolvente del canal izquierdo, por ejemplo */
		envfollower_work(&ef, left_input, env_output);
		/*   ...   */
		/*   aqu hacemos el procesado   */
		/*   ...   */
		buffer_play(fd, left_output, right_output, 1.0);
		}
	envfollower_done(&ef);
	close(fd);

	return 0;
	}
