/*
 * (c) Avelino Herrera Morales
 *
 * Parser de eventos MIDI. Los mensajes MIDI traducidos se ponen en memoria compartida,
 * en la estructura de datos definida en shm_midi.h
 *
	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "shm_midi.h"


/*  /dev/midi file descriptor  */
int fd;
unsigned char data;
/*  automata status  */
int by;
int last_note;
int shmid;
unsigned char ch;

shm_midi *midi_mem;


void process_data(unsigned char d) {
	switch (d & 0xF0) {
		case 0x90 : by = 1;
		            ch = d & 0x0F;
		            return;
		case 0xB0 : by = 11;
		            ch = d & 0x0F;
		            return;
		case 0xC0 : by = 21;
		            ch = d & 0x0F;
		            return;
		case 0xE0 : by = 22;
		            ch = d & 0x0F;
		            return;
		}
	if (by == 1) {
		last_note = d;
		by = 2;
		return;
		}
	else if (by == 2) {
		if (d == 0) {
			midi_mem[ch].keyboard_hit[last_note] = 0;
			printf("[%2d] Note OFF %3d\n", ch, last_note);
			}
		 else {
			midi_mem[ch].keyboard_hit[last_note] = 1;
			midi_mem[ch].keyboard_velocity[last_note] = d;
			printf("[%2d] Note ON  %3d %3d\n", ch, last_note, d);
			}
		by = 1;
		return;
		}
	else if (by == 11) {
		last_note = d;
		by = 12;
		return;
		}
	else if (by == 12) {
		midi_mem[ch].controller[0x7F & last_note] = d;
		printf("[%2d] Controller %3d = %3d\n", ch, last_note, d);
		by = 11;
		return;
		}
	else if (by == 21) {
		midi_mem[ch].program = d;
		printf("[%2d] Program change to %3d\n", ch, d);
		by = 0;
		return;
		}
	else if (by == 22) {
		midi_mem[ch].pitch_bend = d;
		by = 23;
		return;
		}
	else if (by == 23) {
		midi_mem[ch].pitch_bend |= ((int) d << 7);
		printf("[%2d] Pitch wheel change to %d\n", ch, midi_mem[ch].pitch_bend);
		by = 22;
		return;
		}
	}

void signal_handler(int sig) {
	printf("Ctrl-C pressed. Terminating.\n");
	shmdt(midi_mem);
	shmctl(shmid, IPC_RMID, (void *) 0);
	close(fd);
	exit(0);
	}

int main(void) {
	fd = open("/dev/midi", O_RDONLY);
	if (fd < 0) {
		perror("main");
		return -1;
		}
	signal(SIGINT, signal_handler);

	shmid = shmget(SHM_MIDI_KEY, sizeof(shm_midi) * SHM_MIDI_CH,
	               0700 | IPC_CREAT);
	if (shmid < 0) {
		perror("main");
		return -1;
		}
	midi_mem = shmat(shmid, (const void *) 0, SHM_RND);

	by = 0;
	while (1) {
		if (1 != read(fd, &data, 1))
			break;
		if (data == 254)
			continue;
		/* printf("%X\n", data); */
		process_data(data);
		}
	return 0;
	}
