/*
 *    Reverberador JCRev (John Chowning, CCRMA)
 *
 *    Avelino Herrera Morales para "Slo Programadores"

	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>

/* Parmetros de la reverb */
#define  AP1_G    0.7
#define  AP1_M    1051
#define  AP2_G    0.7
#define  AP2_M    337
#define  AP3_G    0.7
#define  AP3_M    113
#define  FFCF1_G  0.742
#define  FFCF1_M  4799
#define  FFCF2_G  0.733
#define  FFCF2_M  4799
#define  FFCF3_G  0.715
#define  FFCF3_M  5399
#define  FFCF4_G  0.697
#define  FFCF4_M  5801


/*  Frecuencia de muestreo de salida. */
#define  SAMPLE_RATE           44100

#ifdef M_PI
#define  PI  M_PI
#else
#define  PI  3.1415926
#endif

/*  Devuelve el nmero de muestras que hay en m milisegundos. */
#define  GET_SAMPLES_MS(m)  ((m * SAMPLE_RATE) / 10000)
/*  Devuelve el nmero de muestras que hay en m centsimas de segundo. */
#define  GET_SAMPLES_CS(m)  ((m * SAMPLE_RATE) / 100)

#define  BUFFER_SIZE  256

typedef struct {
	float allpass1_feedback;
	float *allpass1;
	int allpass1_size;
	int ta1, ta1_out;

	float allpass2_feedback;
	float *allpass2;
	int allpass2_size;
	int ta2, ta2_out;

	float allpass3_feedback;
	float *allpass3;
	int allpass3_size;
	int ta3, ta3_out;

	float comb1_feedback;
	float *comb1;
	int comb1_size;
	int t1, t1_out;

	float comb2_feedback;
	float *comb2;
	int comb2_size;
	int t2, t2_out;

	float comb3_feedback;
	float *comb3;
	int comb3_size;
	int t3, t3_out;

	float comb4_feedback;
	float *comb4;
	int comb4_size;
	int t4, t4_out;

	float mix;
	int only_reverbered;
	} reverb;


void reverb_init(reverb *r, float mix) {
	/* Primer filtro pasa-todo */
	r->allpass1_size = AP1_M;
	r->allpass1 = malloc(r->allpass1_size * sizeof(float));
	if (r->allpass1 == NULL)
		return;
	r->allpass1_feedback = AP1_G;
	r->ta1 = 0;
	r->ta1_out = 1;
	/* Segundo filtro pasa-todo */
	r->allpass2_size = AP2_M;
	r->allpass2 = malloc(r->allpass2_size * sizeof(float));
	if (r->allpass2 == NULL)
		return;
	r->allpass2_feedback = AP2_G;
	r->ta2 = 0;
	r->ta2_out = 1;
	/* Tercer filtro pasa-todo */
	r->allpass3_size = AP3_M;
	r->allpass3 = malloc(r->allpass3_size * sizeof(float));
	if (r->allpass3 == NULL)
		return;
	r->allpass3_feedback = AP3_G;
	r->ta3 = 0;
	r->ta3_out = 1;
	/* Primer filtro peine */
	r->comb1_size = FFCF1_M;
	r->comb1 = malloc(r->comb1_size * sizeof(float));
	if (r->comb1 == NULL)
		return;
	r->comb1_feedback = FFCF1_G;
	r->t1 = 0;
	r->t1_out = 1;
	/* Segundo filtro peine */
	r->comb2_size = FFCF2_M;
	r->comb2 = malloc(r->comb2_size * sizeof(float));
	if (r->comb2 == NULL)
		return;
	r->comb2_feedback = FFCF2_G;
	r->t2 = 0;
	r->t2_out = 1;
	/* Tercer filtro peine */
	r->comb3_size = FFCF3_M;
	r->comb3 = malloc(r->comb3_size * sizeof(float));
	if (r->comb3 == NULL)
		return;
	r->comb3_feedback = FFCF3_G;
	r->t3 = 0;
	r->t3_out = 1;
	/* Cuarto filtro peine */
	r->comb4_size = FFCF4_M;
	r->comb4 = malloc(r->comb4_size * sizeof(float));
	if (r->comb4 == NULL)
		return;
	r->comb4_feedback = FFCF4_G;
	r->t4 = 0;
	r->t4_out = 1;
	/* Mezcla final */
	r->mix = mix;
	return;
	}

void reverb_done(reverb *r) {
	free(r->allpass1);
	free(r->allpass2);
	free(r->allpass3);
	free(r->comb1);
	free(r->comb2);
	free(r->comb3);
	free(r->comb4);
	return;
	}

void reverb_work(reverb *r, float *input, float *output) {
	int i;
	float all1_output, all2_output, all3_output, comb_output;

	for (i = 0; i < BUFFER_SIZE; i++) {
		/* Primer filtro pasa-todo */
		r->allpass1[r->ta1] = input[i] + (r->allpass1[r->ta1_out] * (-r->allpass1_feedback));
		all1_output = (r->allpass1[r->ta1] * r->allpass1_feedback) + r->allpass1[r->ta1_out];
		r->ta1 = (r->ta1 + 1) % r->allpass1_size;
		r->ta1_out = (r->ta1_out + 1) % r->allpass1_size;

		/* Segundo filtro pasa-todo */
		r->allpass2[r->ta2] = all1_output + (r->allpass2[r->ta2_out] * (-r->allpass2_feedback));
		all2_output = (r->allpass2[r->ta2] * r->allpass2_feedback) + r->allpass2[r->ta2_out];
		r->ta2 = (r->ta2 + 1) % r->allpass2_size;
		r->ta2_out = (r->ta2_out + 1) % r->allpass2_size;

		/* Tercer filtro pasa-todo */
		r->allpass3[r->ta3] = all2_output + (r->allpass3[r->ta3_out] * (-r->allpass3_feedback));
		all3_output = (r->allpass3[r->ta3] * r->allpass3_feedback) + r->allpass3[r->ta3_out];
		r->ta3 = (r->ta3 + 1) % r->allpass3_size;
		r->ta3_out = (r->ta3_out + 1) % r->allpass3_size;

		/* Primer filtro peine */
		r->comb1[r->t1] = all3_output;
		comb_output = (r->comb1_feedback * all3_output) + r->comb1[r->t1_out];
		r->t1 = (r->t1 + 1) % r->comb1_size;
		r->t1_out = (r->t1_out + 1) % r->comb1_size;
		/* Segundo filtro peine */
		r->comb2[r->t2] = all3_output;
		comb_output += (r->comb2_feedback * all3_output) + r->comb2[r->t2_out];
		r->t2 = (r->t2 + 1) % r->comb2_size;
		r->t2_out = (r->t2_out + 1) % r->comb2_size;
		/* Tercer filtro peine */
		r->comb3[r->t3] = all3_output;
		comb_output += (r->comb3_feedback * all3_output) + r->comb3[r->t3_out];
		r->t3 = (r->t3 + 1) % r->comb3_size;
		r->t3_out = (r->t3_out + 1) % r->comb3_size;
		/* Cuarto filtro peine */
		r->comb4[r->t4] = all3_output;
		comb_output += (r->comb4_feedback * all3_output) + r->comb4[r->t4_out];
		r->t4 = (r->t4 + 1) % r->comb4_size;
		r->t4_out = (r->t4_out + 1) % r->comb4_size;
		/* Generamos la salida */
		if (r->only_reverbered)
			output[i] = r->mix * comb_output;
		 else
			output[i] = input[i] + (r->mix * comb_output);
		}
	return;
	}


int buffer_rec(int fd, float *left, float *right) {
	int i, readed;
	short int temp_buffer[BUFFER_SIZE * 2];

	readed = read(fd, temp_buffer, BUFFER_SIZE * 2 * sizeof(short int));
	for (i = 0; i < BUFFER_SIZE; i++) {
		left[i] = ((float) temp_buffer[2 * i]) / 32768.0;
		right[i] = ((float) temp_buffer[(2 * i) + 1]) / 32768.0;
		}
	if (readed < (BUFFER_SIZE * 2 * sizeof(short int)))
		return 0;
	return 1;
	}


void buffer_play(int fd, float *left, float *right, float gain) {
	int i;
	int trash;
	short int temp_buffer[BUFFER_SIZE * 2];

	for (i = 0; i < BUFFER_SIZE; i++) {
		left[i] *= gain;
		if (left[i] > 1.0)
			left[i] = 1.0;
		else if (left[i] < -1.0)
			left[i] = -1.0;
		right[i] *= gain;
		if (right[i] > 1.0)
			right[i] = 1.0;
		else if (right[i] < -1.0)
			right[i] = -1.0;
		temp_buffer[2 * i] = (short) rint(left[i] * 32767);
		temp_buffer[(2 * i) + 1] = (short) rint(right[i] * 32767);
		}
	write(fd, temp_buffer, BUFFER_SIZE * 2 * sizeof(short int));
	return;
	}


int audio_open(void) {
	int rate = SAMPLE_RATE;
	int format = AFMT_S16_LE;
	int stereo = 1;
	int fd;

	if ((fd = open("/dev/dsp", O_WRONLY, 0)) == -1) {
		perror("/dev/dsp");
		exit(-1);
		}
	ioctl(fd, SNDCTL_DSP_SPEED, &rate);
	ioctl(fd, SNDCTL_DSP_SETFMT, &format);
	ioctl(fd, SNDCTL_DSP_STEREO, &stereo);
	return fd;
	}


/*****************************************************************************/
int main(int argc, char *argv[]) {
	reverb left_rev;
	reverb right_rev;
	float left_input[BUFFER_SIZE], right_input[BUFFER_SIZE];
	float left_output[BUFFER_SIZE], right_output[BUFFER_SIZE];
	int fd;

	/* Abrimos el audio */
	fd = audio_open();

	/* Inicializamos */
	reverb_init(&left_rev, 0.09);
	reverb_init(&right_rev, 0.09);

	left_rev.only_reverbered = 0;
	right_rev.only_reverbered = 0;

	if ((argc > 1) && (strcmp(argv[1], "-r") == 0)) {
		left_rev.only_reverbered = 1;
		right_rev.only_reverbered = 1;
		}

	/* Bucle principal */
	/* Leemos de la entrada estndar */
	while (buffer_rec(0, left_input, right_input)) {
		reverb_work(&left_rev, left_input, left_output);
		reverb_work(&right_rev, right_input, right_output);
		/* La seal reverberada la enviamos al dispositivo de salida */
		buffer_play(fd, left_output, right_output, 1.0);
		}

	/* Terminamos */
	reverb_done(&left_rev);
	reverb_done(&right_rev);
	close(fd);

	return 0;
	}
