/*
 * (c) Avelino Herrera Morales
 *
 * Programa ejemplo que quita las voces de las canciones estéreo.
 * Se basa en el hecho de que en el "centro" de la grabación sólo se alojan los bajos y las voces principales.
 * Si restamos la info del canal izquierdo menos la del derecho o viceversa, salvaguardando las frecuancias más bajas,
 * conseguiremos un efecto de supresión de voz bastante efectivo.
 *
	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>


/*  Frecuencia de muestreo de salida. */
#define  SAMPLE_RATE    44100
#define  BUFFER_SIZE    256
#define  FILTER_CUTOFF  0.02
#define  FILTER_RESO    0.5


/******************************************************************************
	Record buffers from fd.
*/
int buffer_rec(int fd, float *left, float *right) {
	int i, readed;
	short int temp_buffer[BUFFER_SIZE * 2];

	readed = read(fd, temp_buffer, BUFFER_SIZE * 2 * sizeof(short int));
	for (i = 0; i < BUFFER_SIZE; i++) {
		left[i] = ((float) temp_buffer[2 * i]) / 32768.0;
		right[i] = ((float) temp_buffer[(2 * i) + 1]) / 32768.0;
		}
	if (readed < (BUFFER_SIZE * 2 * sizeof(short int)))
		return 0;
	return 1;
	}

/******************************************************************************
	Play buffers throw fd.
*/
void buffer_play(int fd, float *left, float *right, float gain) {
	int i;
	int trash;
	short int temp_buffer[BUFFER_SIZE * 2];

	for (i = 0; i < BUFFER_SIZE; i++) {
		left[i] *= gain;
		if (left[i] > 1.0)
			left[i] = 1.0;
		else if (left[i] < -1.0)
			left[i] = -1.0;
		right[i] *= gain;
		if (right[i] > 1.0)
			right[i] = 1.0;
		else if (right[i] < -1.0)
			right[i] = -1.0;
		temp_buffer[2 * i] = (short) rint(left[i] * 32767);
		temp_buffer[(2 * i) + 1] = (short) rint(right[i] * 32767);
		}
	write(fd, temp_buffer, BUFFER_SIZE * 2 * sizeof(short int));
	return;
	}

/******************************************************************************
	Open /dev/dsp for listen.
*/
int audio_open(void) {
	int rate = SAMPLE_RATE;
	int format = AFMT_S16_LE;
	int stereo = 2;
	int fd;

	if ((fd = open("/dev/dsp", O_WRONLY, 0)) == -1) {
		perror("/dev/dsp");
		exit(-1);
		}
	ioctl(fd, SNDCTL_DSP_SPEED, &rate);
	ioctl(fd, SNDCTL_DSP_SETFMT, &format);
	ioctl(fd, SNDCTL_DSP_STEREO, &stereo);
	return fd;
	}

/*****************************************************************************/
int main(int argc, char *argv[]) {
	int fd, i;
	float left_input[BUFFER_SIZE], right_input[BUFFER_SIZE];
	float left_output[BUFFER_SIZE], right_output[BUFFER_SIZE];
	float add, sub, out;
	float hig1 = 0, mid1 = 0, low1 = 0;
	float hig2 = 0, mid2 = 0, low2 = 0;

	fd = audio_open();
	while (buffer_rec(0, left_input, right_input)) {
		for (i = 0; i < BUFFER_SIZE; i++) {
			add = (left_input[i] + right_input[i]) / 2;
			hig1 = add - (mid1 * FILTER_RESO) - low1;
			mid1 += hig1 * FILTER_CUTOFF;
			low1 += mid1 * FILTER_CUTOFF;
			hig2 = low1 - (mid2 * FILTER_RESO) - low2;
			mid2 += hig2 * FILTER_CUTOFF;
			low2 += mid2 * FILTER_CUTOFF;
			sub = left_input[i] - right_input[i];
			left_output[i] = low2 + sub;
			right_output[i] = low2 + sub;
			}
		buffer_play(fd, left_output, right_output, 1.0);
		}
	close(fd);

	return 0;
	}
